/**************************************************************************
*  Copyright (c) 2013 by Michael Fischer. All rights reserved.
*
*  Based on a template from ChibiOS/RT. Therefore partial copyright:
*  Copyright (c) 2006-2012, Giovanni Di Sirio
*
*  Based on an StarterWare example from TI. Therefore partial copyright:
*  Copyright (c) 2010 Texas Instruments Incorporated
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  18.02.2013 mifi  First Version for the BeageBone board.
*                   CrossWorks for ARM v2.3.0 was used.
**************************************************************************/

/*=======================================================================*/
/*  Include                                                              */
/*=======================================================================*/
#include "ch.h"
#include "hal.h"

#include "dmtimer.h"
#include "interrupt.h"

#include "hw_control_AM335x.h"
#include "soc_AM335x.h"
#include "hw_cm_wkup.h"
#include "hw_cm_per.h"
#include "hw_cm_dpll.h"
#include "hw_types.h"

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

/*=======================================================================*/
/*  Prototypes                                                           */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*************************************************************************/
/*  TimerTickIrqHandler                                                  */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static CH_IRQ_HANDLER (TimerTickIrqHandler) 
{
  CH_IRQ_PROLOGUE();

  chSysLockFromIsr();
  chSysTimerHandlerI();
  chSysUnlockFromIsr();
   
  /* Clear the status of the interrupt flags */
  DMTimerIntStatusClear(SOC_DMTIMER_2_REGS, DMTIMER_INT_OVF_IT_FLAG);
   
  CH_IRQ_EPILOGUE();
} /* TimerTickIrqHandler */

/*************************************************************************/
/*  InitTimerTick                                                        */
/*                                                                       */
/* (Based on the DMTimer2ModuleClkConfig function from the BeagleBone)   */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void InitTimerTick (void)
{
  uint32_t counter;
  
  /* 
   * Select the clock source for the Timer2 instance. 
   */
  HWREG(SOC_CM_DPLL_REGS + CM_DPLL_CLKSEL_TIMER2_CLK) &=
        ~(CM_DPLL_CLKSEL_TIMER2_CLK_CLKSEL);

  HWREG(SOC_CM_DPLL_REGS + CM_DPLL_CLKSEL_TIMER2_CLK) |=
        CM_DPLL_CLKSEL_TIMER2_CLK_CLKSEL_CLK_M_OSC;

  while((HWREG(SOC_CM_DPLL_REGS + CM_DPLL_CLKSEL_TIMER2_CLK) &
         CM_DPLL_CLKSEL_TIMER2_CLK_CLKSEL) !=
         CM_DPLL_CLKSEL_TIMER2_CLK_CLKSEL_CLK_M_OSC);

  /* Writing to MODULEMODE field of CM_PER_TIMER2_CLKCTRL register. */
  HWREG(SOC_CM_PER_REGS + CM_PER_TIMER2_CLKCTRL) |=
        CM_PER_TIMER2_CLKCTRL_MODULEMODE_ENABLE;

  /* Waiting for MODULEMODE field to reflect the written value. */
  while((HWREG(SOC_CM_PER_REGS + CM_PER_TIMER2_CLKCTRL) &
         CM_PER_TIMER2_CLKCTRL_MODULEMODE) != 
         CM_PER_TIMER2_CLKCTRL_MODULEMODE_ENABLE);

  /*
   * Waiting for IDLEST field in CM_PER_TIMER2_CLKCTRL register 
   * for the module is fully functional.
   */
  while((HWREG(SOC_CM_PER_REGS + CM_PER_TIMER2_CLKCTRL) & 
         CM_PER_TIMER2_CLKCTRL_IDLEST) != 
         CM_PER_TIMER2_CLKCTRL_IDLEST_FUNC);
   
  /* Waiting for the L4LS clock */
  while(!(HWREG(SOC_CM_PER_REGS + CM_PER_L4LS_CLKSTCTRL) &
          CM_PER_L4LS_CLKSTCTRL_CLKACTIVITY_L4LS_GCLK));

  /* Waiting for the TIMER2 clock */
  while(!(HWREG(SOC_CM_PER_REGS + CM_PER_L4LS_CLKSTCTRL) &
          CM_PER_L4LS_CLKSTCTRL_CLKACTIVITY_TIMER2_GCLK));


  /*
   * Setup timer.
   */
   
  /* Registering DMTimerIsr */
  IntRegister(SYS_INT_TINT2, TimerTickIrqHandler);

  /* Set the priority */
  IntPrioritySet(SYS_INT_TINT2, 
                 AM335X_HAL_TIME_TICK_PRIORITY, AINTC_HOSTINT_ROUTE_IRQ);

  /* Enable the system interrupt */
  IntSystemEnable(SYS_INT_TINT2);

  /* Setup timer */
  counter = 0xffffffff - (24000000UL / CH_FREQUENCY);
  
  DMTimerReset(SOC_DMTIMER_2_REGS);
  DMTimerCounterSet(SOC_DMTIMER_2_REGS, counter);
  DMTimerReloadSet(SOC_DMTIMER_2_REGS, counter);
  DMTimerModeConfigure(SOC_DMTIMER_2_REGS, DMTIMER_AUTORLD_NOCMP_ENABLE);
  
  /* Start timer */
  DMTimerEnable(SOC_DMTIMER_2_REGS);   
  
  /* Enable timer interrupt */
  DMTimerIntEnable(SOC_DMTIMER_2_REGS, DMTIMER_INT_OVF_EN_FLAG);
  
} /* InitTimerTick */

/*************************************************************************/
/*  InitL3WakeUpClock                                                    */
/*                                                                       */
/*  Init L3 and Wake-Up clock.                                           */
/*  (Based on the TI platform\baglebone\uart example)                    */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void InitL3WakeUpClock (void)
{
  /* Configuring L3 Interface Clocks. */

  /* Writing to MODULEMODE field of CM_PER_L3_CLKCTRL register. */
  HWREG(SOC_CM_PER_REGS + CM_PER_L3_CLKCTRL) |=
        CM_PER_L3_CLKCTRL_MODULEMODE_ENABLE;

  /* Waiting for MODULEMODE field to reflect the written value. */
  while(CM_PER_L3_CLKCTRL_MODULEMODE_ENABLE !=
        (HWREG(SOC_CM_PER_REGS + CM_PER_L3_CLKCTRL) &
         CM_PER_L3_CLKCTRL_MODULEMODE));

  /* Writing to MODULEMODE field of CM_PER_L3_INSTR_CLKCTRL register. */
  HWREG(SOC_CM_PER_REGS + CM_PER_L3_INSTR_CLKCTRL) |=
        CM_PER_L3_INSTR_CLKCTRL_MODULEMODE_ENABLE;

  /* Waiting for MODULEMODE field to reflect the written value. */
  while(CM_PER_L3_INSTR_CLKCTRL_MODULEMODE_ENABLE !=
        (HWREG(SOC_CM_PER_REGS + CM_PER_L3_INSTR_CLKCTRL) &
         CM_PER_L3_INSTR_CLKCTRL_MODULEMODE));

  /* Writing to CLKTRCTRL field of CM_PER_L3_CLKSTCTRL register. */
  HWREG(SOC_CM_PER_REGS + CM_PER_L3_CLKSTCTRL) |=
        CM_PER_L3_CLKSTCTRL_CLKTRCTRL_SW_WKUP;

  /* Waiting for CLKTRCTRL field to reflect the written value. */
  while(CM_PER_L3_CLKSTCTRL_CLKTRCTRL_SW_WKUP !=
        (HWREG(SOC_CM_PER_REGS + CM_PER_L3_CLKSTCTRL) &
         CM_PER_L3_CLKSTCTRL_CLKTRCTRL));

  /* Writing to CLKTRCTRL field of CM_PER_OCPWP_L3_CLKSTCTRL register. */
  HWREG(SOC_CM_PER_REGS + CM_PER_OCPWP_L3_CLKSTCTRL) |=
        CM_PER_OCPWP_L3_CLKSTCTRL_CLKTRCTRL_SW_WKUP;

  /*Waiting for CLKTRCTRL field to reflect the written value. */
  while(CM_PER_OCPWP_L3_CLKSTCTRL_CLKTRCTRL_SW_WKUP !=
        (HWREG(SOC_CM_PER_REGS + CM_PER_OCPWP_L3_CLKSTCTRL) &
         CM_PER_OCPWP_L3_CLKSTCTRL_CLKTRCTRL));

  /* Writing to CLKTRCTRL field of CM_PER_L3S_CLKSTCTRL register. */
  HWREG(SOC_CM_PER_REGS + CM_PER_L3S_CLKSTCTRL) |=
        CM_PER_L3S_CLKSTCTRL_CLKTRCTRL_SW_WKUP;

  /*Waiting for CLKTRCTRL field to reflect the written value. */
  while(CM_PER_L3S_CLKSTCTRL_CLKTRCTRL_SW_WKUP !=
        (HWREG(SOC_CM_PER_REGS + CM_PER_L3S_CLKSTCTRL) &
         CM_PER_L3S_CLKSTCTRL_CLKTRCTRL));

  /* Checking fields for necessary values.  */

  /* Waiting for IDLEST field in CM_PER_L3_CLKCTRL register to be set to 0x0. */
  while((CM_PER_L3_CLKCTRL_IDLEST_FUNC << CM_PER_L3_CLKCTRL_IDLEST_SHIFT)!=
        (HWREG(SOC_CM_PER_REGS + CM_PER_L3_CLKCTRL) &
         CM_PER_L3_CLKCTRL_IDLEST));

  /*
  ** Waiting for IDLEST field in CM_PER_L3_INSTR_CLKCTRL register to attain the
  ** desired value.
  */
  while((CM_PER_L3_INSTR_CLKCTRL_IDLEST_FUNC <<
         CM_PER_L3_INSTR_CLKCTRL_IDLEST_SHIFT)!=
        (HWREG(SOC_CM_PER_REGS + CM_PER_L3_INSTR_CLKCTRL) &
         CM_PER_L3_INSTR_CLKCTRL_IDLEST));

  /*
  ** Waiting for CLKACTIVITY_L3_GCLK field in CM_PER_L3_CLKSTCTRL register to
  ** attain the desired value.
  */
  while(CM_PER_L3_CLKSTCTRL_CLKACTIVITY_L3_GCLK !=
        (HWREG(SOC_CM_PER_REGS + CM_PER_L3_CLKSTCTRL) &
         CM_PER_L3_CLKSTCTRL_CLKACTIVITY_L3_GCLK));

  /*
  ** Waiting for CLKACTIVITY_OCPWP_L3_GCLK field in CM_PER_OCPWP_L3_CLKSTCTRL
  ** register to attain the desired value.
  */
  while(CM_PER_OCPWP_L3_CLKSTCTRL_CLKACTIVITY_OCPWP_L3_GCLK !=
        (HWREG(SOC_CM_PER_REGS + CM_PER_OCPWP_L3_CLKSTCTRL) &
         CM_PER_OCPWP_L3_CLKSTCTRL_CLKACTIVITY_OCPWP_L3_GCLK));

  /*
  ** Waiting for CLKACTIVITY_L3S_GCLK field in CM_PER_L3S_CLKSTCTRL register
  ** to attain the desired value.
  */
  while(CM_PER_L3S_CLKSTCTRL_CLKACTIVITY_L3S_GCLK !=
        (HWREG(SOC_CM_PER_REGS + CM_PER_L3S_CLKSTCTRL) &
        CM_PER_L3S_CLKSTCTRL_CLKACTIVITY_L3S_GCLK));


  /* Configuring registers related to Wake-Up region. */

  /* Writing to MODULEMODE field of CM_WKUP_CONTROL_CLKCTRL register. */
  HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CONTROL_CLKCTRL) |=
        CM_WKUP_CONTROL_CLKCTRL_MODULEMODE_ENABLE;

  /* Waiting for MODULEMODE field to reflect the written value. */
  while(CM_WKUP_CONTROL_CLKCTRL_MODULEMODE_ENABLE !=
        (HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CONTROL_CLKCTRL) &
         CM_WKUP_CONTROL_CLKCTRL_MODULEMODE));

  /* Writing to CLKTRCTRL field of CM_PER_L3S_CLKSTCTRL register. */
  HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CLKSTCTRL) |=
        CM_WKUP_CLKSTCTRL_CLKTRCTRL_SW_WKUP;

  /*Waiting for CLKTRCTRL field to reflect the written value. */
  while(CM_WKUP_CLKSTCTRL_CLKTRCTRL_SW_WKUP !=
        (HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CLKSTCTRL) &
         CM_WKUP_CLKSTCTRL_CLKTRCTRL));

  /* Writing to CLKTRCTRL field of CM_L3_AON_CLKSTCTRL register. */
  HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_L3_AON_CLKSTCTRL) |=
        CM_WKUP_CM_L3_AON_CLKSTCTRL_CLKTRCTRL_SW_WKUP;

  /*Waiting for CLKTRCTRL field to reflect the written value. */
  while(CM_WKUP_CM_L3_AON_CLKSTCTRL_CLKTRCTRL_SW_WKUP !=
        (HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_L3_AON_CLKSTCTRL) &
         CM_WKUP_CM_L3_AON_CLKSTCTRL_CLKTRCTRL));

  /* Verifying if the other bits are set to required settings. */

  /*
  ** Waiting for IDLEST field in CM_WKUP_CONTROL_CLKCTRL register to attain
  ** desired value.
  */
  while((CM_WKUP_CONTROL_CLKCTRL_IDLEST_FUNC <<
         CM_WKUP_CONTROL_CLKCTRL_IDLEST_SHIFT) !=
        (HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CONTROL_CLKCTRL) &
         CM_WKUP_CONTROL_CLKCTRL_IDLEST));

  /*
  ** Waiting for CLKACTIVITY_L3_AON_GCLK field in CM_L3_AON_CLKSTCTRL
  ** register to attain desired value.
  */
  while(CM_WKUP_CM_L3_AON_CLKSTCTRL_CLKACTIVITY_L3_AON_GCLK !=
        (HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_L3_AON_CLKSTCTRL) &
         CM_WKUP_CM_L3_AON_CLKSTCTRL_CLKACTIVITY_L3_AON_GCLK));

  /*
  ** Waiting for IDLEST field in CM_WKUP_L4WKUP_CLKCTRL register to attain
  ** desired value.
  */
  while((CM_WKUP_L4WKUP_CLKCTRL_IDLEST_FUNC <<
         CM_WKUP_L4WKUP_CLKCTRL_IDLEST_SHIFT) !=
        (HWREG(SOC_CM_WKUP_REGS + CM_WKUP_L4WKUP_CLKCTRL) &
         CM_WKUP_L4WKUP_CLKCTRL_IDLEST));

  /*
  ** Waiting for CLKACTIVITY_L4_WKUP_GCLK field in CM_WKUP_CLKSTCTRL register
  ** to attain desired value.
  */
  while(CM_WKUP_CLKSTCTRL_CLKACTIVITY_L4_WKUP_GCLK !=
        (HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CLKSTCTRL) &
         CM_WKUP_CLKSTCTRL_CLKACTIVITY_L4_WKUP_GCLK));

  /*
  ** Waiting for CLKACTIVITY_L4_WKUP_AON_GCLK field in CM_L4_WKUP_AON_CLKSTCTRL
  ** register to attain desired value.
  */
  while(CM_WKUP_CM_L4_WKUP_AON_CLKSTCTRL_CLKACTIVITY_L4_WKUP_AON_GCLK !=
        (HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CM_L4_WKUP_AON_CLKSTCTRL) &
         CM_WKUP_CM_L4_WKUP_AON_CLKSTCTRL_CLKACTIVITY_L4_WKUP_AON_GCLK));
         
} /* InitL3WakeUpClock */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  hal_lld_init                                                         */
/*                                                                       */
/*  Low level HAL driver initialization.                                 */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void hal_lld_init (void)
{
  /*
   * Init L3 and Wake-Up clock
   */
  InitL3WakeUpClock();
  
  /*
   * Setup the SysTick timer
   */
  InitTimerTick();   

} /* hal_lld_init */

/*** EOF ***/


