/**************************************************************************
*  Copyright (c) 2013 by Michael Fischer. All rights reserved.
*
*  Based on a template from ChibiOS/RT. Therefore partial copyright:
*  Copyright (c) 2006-2012, Giovanni Di Sirio.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  18.02.2013 mifi  First Version for the BeageBone board.
*                   CrossWorks for ARM v2.3.0 was used.
**************************************************************************/

/*=======================================================================*/
/*  Include                                                              */
/*=======================================================================*/
#include "ch.h"
#include "hal.h"

#if HAL_USE_PAL || defined(__DOXYGEN__)

#include "soc_AM335x.h"
#include "hw_control_AM335x.h"
#include "hw_types.h"
#include "hw_cm_per.h"
#include "hw_gpio_v2.h"

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

/*
 * Some defines for the control configuration
 */
#define CONTROL_CONF_MASK   0x7F

/*=======================================================================*/
/*  Prototypes                                                           */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

static const uint32_t GPIO_0_OFFSET[32] =
{
  CONTROL_CONF_MDIO_DATA,           /* GPIO0_0  */
  CONTROL_CONF_MDIO_CLK,            /* GPIO0_1  */
  CONTROL_CONF_SPI0_SCLK,           /* GPIO0_2  */
  CONTROL_CONF_SPI0_D0,             /* GPIO0_3  */
  CONTROL_CONF_SPI0_D1,             /* GPIO0_4  */
  CONTROL_CONF_SPI0_CS0,            /* GPIO0_5  */
  CONTROL_CONF_SPI0_CS1,            /* GPIO0_6  */
  CONTROL_CONF_ECAP0_IN_PWM0_OUT,   /* GPIO0_7  */
  CONTROL_CONF_LCD_DATA(12),        /* GPIO0_8  */
  CONTROL_CONF_LCD_DATA(13),        /* GPIO0_9  */
  CONTROL_CONF_LCD_DATA(14),        /* GPIO0_10 */
  CONTROL_CONF_LCD_DATA(15),        /* GPIO0_11 */
  CONTROL_CONF_UART_CTSN(1),        /* GPIO0_12 */
  CONTROL_CONF_UART_RTSN(1),        /* GPIO0_13 */
  CONTROL_CONF_UART_RXD(1),         /* GPIO0_14 */
  CONTROL_CONF_UART_TXD(1),         /* GPIO0_15 */
  CONTROL_CONF_MII1_TXD3,           /* GPIO0_16 */
  CONTROL_CONF_MII1_TXD2,           /* GPIO0_17 */
  CONTROL_CONF_USB_DRVVBUS(0),      /* GPIO0_18 */
  CONTROL_CONF_XDMA_EVENT_INTR(0),  /* GPIO0_19 */
  CONTROL_CONF_XDMA_EVENT_INTR(1),  /* GPIO0_20 */
  CONTROL_CONF_MII1_TXD1,           /* GPIO0_21 */
  CONTROL_CONF_GPMC_AD(8),          /* GPIO0_22 */
  CONTROL_CONF_GPMC_AD(9),          /* GPIO0_23 */
  0,                                /* GPIO0_24 */
  0,                                /* GPIO0_25 */
  CONTROL_CONF_GPMC_AD(10),         /* GPIO0_26 */
  CONTROL_CONF_GPMC_AD(11),         /* GPIO0_27 */
  CONTROL_CONF_MII1_TXD0,           /* GPIO0_28 */
  CONTROL_CONF_RMII1_REFCLK,        /* GPIO0_29 */
  CONTROL_CONF_GPMC_WAIT0,          /* GPIO0_30 */
  CONTROL_CONF_GPMC_WPN             /* GPIO0_31 */
};

static const uint32_t GPIO_1_OFFSET[32] =
{
  CONTROL_CONF_GPMC_AD(0),          /* GPIO1_0  */
  CONTROL_CONF_GPMC_AD(1),          /* GPIO1_1  */
  CONTROL_CONF_GPMC_AD(2),          /* GPIO1_2  */
  CONTROL_CONF_GPMC_AD(3),          /* GPIO1_3  */
  CONTROL_CONF_GPMC_AD(4),          /* GPIO1_4  */
  CONTROL_CONF_GPMC_AD(5),          /* GPIO1_5  */
  CONTROL_CONF_GPMC_AD(6),          /* GPIO1_6  */
  CONTROL_CONF_GPMC_AD(7),          /* GPIO1_7  */
  CONTROL_CONF_UART_CTSN(0),        /* GPIO1_8  */
  CONTROL_CONF_UART_RTSN(0),        /* GPIO1_9  */
  CONTROL_CONF_UART_RXD(0),         /* GPIO1_10 */
  CONTROL_CONF_UART_TXD(0),         /* GPIO1_11 */
  CONTROL_CONF_GPMC_AD(12),         /* GPIO1_12 */
  CONTROL_CONF_GPMC_AD(13),         /* GPIO1_13 */
  CONTROL_CONF_GPMC_AD(14),         /* GPIO1_14 */
  CONTROL_CONF_GPMC_AD(15),         /* GPIO1_15 */
  CONTROL_CONF_GPMC_A(0),           /* GPIO1_16 */
  CONTROL_CONF_GPMC_A(1),           /* GPIO1_17 */
  CONTROL_CONF_GPMC_A(2),           /* GPIO1_18 */
  CONTROL_CONF_GPMC_A(3),           /* GPIO1_19 */
  CONTROL_CONF_GPMC_A(4),           /* GPIO1_20 */
  CONTROL_CONF_GPMC_A(5),           /* GPIO1_21 */
  CONTROL_CONF_GPMC_A(6),           /* GPIO1_22 */
  CONTROL_CONF_GPMC_A(7),           /* GPIO1_23 */
  CONTROL_CONF_GPMC_A(8),           /* GPIO1_24 */
  CONTROL_CONF_GPMC_A(9),           /* GPIO1_25 */
  CONTROL_CONF_GPMC_A(10),          /* GPIO1_26 */
  CONTROL_CONF_GPMC_A(11),          /* GPIO1_27 */
  CONTROL_CONF_GPMC_BE1N,           /* GPIO1_28 */
  CONTROL_CONF_GPMC_CSN(0),         /* GPIO1_29 */
  CONTROL_CONF_GPMC_CSN(1),         /* GPIO1_30 */
  CONTROL_CONF_GPMC_CSN(2)          /* GPIO1_31 */
};                        


static const uint32_t GPIO_2_OFFSET[32] =
{
  CONTROL_CONF_GPMC_CSN(3),         /* GPIO2_0  */
  CONTROL_CONF_GPMC_CLK,            /* GPIO2_1  */
  CONTROL_CONF_GPMC_ADVN_ALE,       /* GPIO2_2  */
  CONTROL_CONF_GPMC_OEN_REN,        /* GPIO2_3  */
  CONTROL_CONF_GPMC_WEN ,           /* GPIO2_4  */
  CONTROL_CONF_GPMC_BE0N_CLE,       /* GPIO2_5  */
  CONTROL_CONF_LCD_DATA(0),         /* GPIO2_6  */
  CONTROL_CONF_LCD_DATA(1),         /* GPIO2_7  */
  CONTROL_CONF_LCD_DATA(2),         /* GPIO2_8  */
  CONTROL_CONF_LCD_DATA(3),         /* GPIO2_9  */
  CONTROL_CONF_LCD_DATA(4),         /* GPIO2_10 */
  CONTROL_CONF_LCD_DATA(5),         /* GPIO2_11 */
  CONTROL_CONF_LCD_DATA(6),         /* GPIO2_12 */
  CONTROL_CONF_LCD_DATA(7),         /* GPIO2_13 */
  CONTROL_CONF_LCD_DATA(8),         /* GPIO2_14 */
  CONTROL_CONF_LCD_DATA(9),         /* GPIO2_15 */
  CONTROL_CONF_LCD_DATA(10),        /* GPIO2_16 */
  CONTROL_CONF_LCD_DATA(11),        /* GPIO2_17 */
  CONTROL_CONF_MII1_RXD3,           /* GPIO2_18 */
  CONTROL_CONF_MII1_RXD2,           /* GPIO2_19 */
  CONTROL_CONF_MII1_RXD1,           /* GPIO2_20 */
  CONTROL_CONF_MII1_RXD0,           /* GPIO2_21 */
  CONTROL_CONF_LCD_VSYNC,           /* GPIO2_22 */
  CONTROL_CONF_LCD_HSYNC,           /* GPIO2_23 */
  CONTROL_CONF_LCD_PCLK,            /* GPIO2_24 */
  CONTROL_CONF_LCD_AC_BIAS_EN,      /* GPIO2_25 */
  CONTROL_CONF_MMC0_DAT3,           /* GPIO2_26 */
  CONTROL_CONF_MMC0_DAT2,           /* GPIO2_27 */
  CONTROL_CONF_MMC0_DAT1,           /* GPIO2_28 */
  CONTROL_CONF_MMC0_DAT0,           /* GPIO2_29 */
  CONTROL_CONF_MMC0_CLK,            /* GPIO2_30 */
  CONTROL_CONF_MMC0_CMD             /* GPIO2_31 */
};  

static const uint32_t GPIO_3_OFFSET[32] =
{
  CONTROL_CONF_MII1_COL,            /* GPIO3_0  */
  CONTROL_CONF_MII1_CRS,            /* GPIO3_1  */
  CONTROL_CONF_MII1_RXERR,          /* GPIO3_2  */
  CONTROL_CONF_MII1_TXEN,           /* GPIO3_3  */
  CONTROL_CONF_MII1_RXDV,           /* GPIO3_4  */
  CONTROL_CONF_I2C0_SDA,            /* GPIO3_5  */
  CONTROL_CONF_I2C0_SCL,            /* GPIO3_6  */
  CONTROL_CONF_EMU(0),              /* GPIO3_7  */
  CONTROL_CONF_EMU(1),              /* GPIO3_8  */
  CONTROL_CONF_MII1_TXCLK,          /* GPIO3_9  */
  CONTROL_CONF_MII1_RXCLK,          /* GPIO3_10 */
  0,                                /* GPIO3_11 */
  0,                                /* GPIO3_12 */
  CONTROL_CONF_USB_DRVVBUS(1),      /* GPIO3_13 */
  CONTROL_CONF_MCASP0_ACLKX,        /* GPIO3_14 */
  CONTROL_CONF_MCASP0_FSX,          /* GPIO3_15 */
  CONTROL_CONF_MCASP0_AXR0,         /* GPIO3_16 */
  CONTROL_CONF_MCASP0_AHCLKR,       /* GPIO3_17 */
  CONTROL_CONF_MCASP0_ACLKR,        /* GPIO3_18 */
  CONTROL_CONF_MCASP0_FSR,          /* GPIO3_19 */
  CONTROL_CONF_MCASP0_AXR1,         /* GPIO3_20 */
  CONTROL_CONF_MCASP0_AHCLKX        /* GPIO3_21 */
};  

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*************************************************************************/
/*  EnableGpio0Clock                                                     */
/*                                                                       */
/*  Enable GPIO clock if it is not still enabled.                        */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void EnableGpio0Clock (void)
{
  static uint16_t Enabled = 0;
  
  if (0 == Enabled)
  {
    Enabled = 1;
    
    /* @@MF TODO */
  }
} /* InitGpio0Clock */

/*************************************************************************/
/*  EnableGpio1Clock                                                     */
/*                                                                       */
/*  Enable GPIO clock if it is not still enabled.                        */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void EnableGpio1Clock (void)
{
  static uint16_t Enabled = 0;
  
  if (0 == Enabled)
  {
    Enabled = 1;

    /* Enable the GPIO1 module */
    HWREG(SOC_PRCM_REGS + CM_PER_GPIO1_CLKCTRL) = 
          CM_PER_GPIO1_CLKCTRL_OPTFCLKEN_GPIO_1_GDBCLK |
          CM_PER_GPIO1_CLKCTRL_MODULEMODE_ENABLE;
        
    /*
     * Waiting for IDLEST field in CM_PER_GPIO1_CLKCTRL register 
     * for the module is fully functional.
     */
    while((HWREG(SOC_CM_PER_REGS + CM_PER_GPIO1_CLKCTRL) & 
           CM_PER_GPIO1_CLKCTRL_IDLEST) != 
           CM_PER_GPIO1_CLKCTRL_IDLEST_FUNC);
           
    /* Module is enabled, clocks are not gated. */
    HWREG(SOC_GPIO_1_REGS + GPIO_CTRL) &= ~GPIO_CTRL_DISABLEMODULE;
  }
} /* InitGpio1Clock */

/*************************************************************************/
/*  EnableGpio2Clock                                                     */
/*                                                                       */
/*  Enable GPIO clock if it is not still enabled.                        */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void EnableGpio2Clock (void)
{
  static uint16_t Enabled = 0;
  
  if (0 == Enabled)
  {
    Enabled = 1;

    /* Enable the GPIO2 module */
    HWREG(SOC_PRCM_REGS + CM_PER_GPIO2_CLKCTRL) = 
          CM_PER_GPIO2_CLKCTRL_OPTFCLKEN_GPIO_2_GDBCLK |
          CM_PER_GPIO2_CLKCTRL_MODULEMODE_ENABLE;
        
    /*
     * Waiting for IDLEST field in CM_PER_GPIO1_CLKCTRL register 
     * for the module is fully functional.
     */
    while((HWREG(SOC_CM_PER_REGS + CM_PER_GPIO2_CLKCTRL) & 
           CM_PER_GPIO2_CLKCTRL_IDLEST) != 
           CM_PER_GPIO2_CLKCTRL_IDLEST_FUNC);
           
    /* Module is enabled, clocks are not gated. */
    HWREG(SOC_GPIO_2_REGS + GPIO_CTRL) &= ~GPIO_CTRL_DISABLEMODULE;
  }
} /* InitGpio2Clock */

/*************************************************************************/
/*  EnableGpio3Clock                                                     */
/*                                                                       */
/*  Enable GPIO clock if it is not still enabled.                        */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void EnableGpio3Clock (void)
{
  static uint16_t Enabled = 0;
  
  if (0 == Enabled)
  {
    Enabled = 1;

    /* Enable the GPIO3 module */
    HWREG(SOC_PRCM_REGS + CM_PER_GPIO3_CLKCTRL) = 
          CM_PER_GPIO3_CLKCTRL_OPTFCLKEN_GPIO_3_GDBCLK |
          CM_PER_GPIO3_CLKCTRL_MODULEMODE_ENABLE;
        
    /*
     * Waiting for IDLEST field in CM_PER_GPIO1_CLKCTRL register 
     * for the module is fully functional.
     */
    while((HWREG(SOC_CM_PER_REGS + CM_PER_GPIO3_CLKCTRL) & 
           CM_PER_GPIO3_CLKCTRL_IDLEST) != 
           CM_PER_GPIO3_CLKCTRL_IDLEST_FUNC);
           
    /* Module is enabled, clocks are not gated. */
    HWREG(SOC_GPIO_3_REGS + GPIO_CTRL) &= ~GPIO_CTRL_DISABLEMODULE;
  }
} /* InitGpio3Clock */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  _pal_lld_init                                                        */
/*                                                                       */
/*  Low level PAL driver initialization.                                 */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void _pal_lld_init (void) 
{
  EnableGpio0Clock();
  EnableGpio1Clock();
  EnableGpio2Clock();
  EnableGpio3Clock();
} /* _pal_lld_init */ 

/*************************************************************************/
/*  _pal_lld_setpadmode                                                  */
/*                                                                       */
/*  Set the Pad Control Registers                                        */
/*                                                                       */
/*  In    : port, pad, mode                                              */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void _pal_lld_setpadmode (ioportid_t port, uint8_t pad, uint32_t mode)
{
  uint32_t ControlValue;
  uint32_t ControlOffset;
  uint32_t Error = TRUE;

  /* Check for correct range: 0...31 */
  if (pad <= 31)
  {
    ControlValue = mode & CONTROL_CONF_MASK;

    switch (port)
    {
      case SOC_GPIO_0_REGS:   /* Valid pin range: 0...23 26...31 */
      { 
        EnableGpio0Clock();
        
        if ((pad <= 23) ||    /* <=  0...23 */ 
            (pad >= 26) )     /* <= 26...31 */
        {
          ControlOffset = GPIO_0_OFFSET[pad];
          Error = FALSE;
        }  
        break;
      } /* SOC_GPIO_0_REGS */
      
      case SOC_GPIO_1_REGS:   /* Valid pin range: 0...31 */
      { 
        EnableGpio1Clock();

        ControlOffset = GPIO_1_OFFSET[pad];
        Error = FALSE;
        break;
      } /* SOC_GPIO_1_REGS */
      
      case SOC_GPIO_2_REGS:   /* Valid pin range: 0...31 */
      { 
        EnableGpio2Clock();
        
        ControlOffset = GPIO_2_OFFSET[pad];
        Error = FALSE;
        break;
      } /* SOC_GPIO_2_REGS */
      
      case SOC_GPIO_3_REGS:   /* Valid pin range: 0...10 13...21 */
      { 
        EnableGpio3Clock();
        
        if ((pad <= 10)                  ||  /* <=  0...10 */ 
            ((pad >= 13) && (pad <= 21)) )   /* <= 13...21 */
        {
          ControlOffset = GPIO_3_OFFSET[pad];
          Error = FALSE;
        }  
        break;
      } /* SOC_GPIO_3_REGS */
      
      default:
      {
        Error = TRUE;
        break;
      }
    } /* end switch (port) */
    
    if (FALSE == Error)
    {
      /* Set the pad control register */
      HWREG(SOC_CONTROL_REGS + ControlOffset) = ControlValue;
        
      if (mode & PAL_MODE_DIR_OUT)
      {
        /* The corresponding GPIO port is configured as an output. */
        HWREG(port + GPIO_OE) &= ~PAL_PORT_BIT(pad);
      }
      else
      {
        /* The corresponding GPIO port is configured as an input. */
        HWREG(port + GPIO_OE) |= PAL_PORT_BIT(pad);
      }
    }/* end if (0 == Error) */
    
  } /* end if (pad <= 31) */

} /* _pal_lld_setpadmode */

#endif /* HAL_USE_PAL */
/*** EOF ***/
