/**************************************************************************
*  Copyright (c) 2013 by Michael Fischer. All rights reserved.
*
*  Based on a template from ChibiOS/RT. Therefore partial copyright:
*  Copyright (c) 2006-2012, Giovanni Di Sirio.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  18.02.2013 mifi  First Version for the BeageBone board.
*                   CrossWorks for ARM v2.3.0 was used.
**************************************************************************/
#ifndef _SERIAL_LLD_H_
#define _SERIAL_LLD_H_

#if HAL_USE_SERIAL || defined(__DOXYGEN__)

/*=======================================================================*/
/* Driver constants.                                                     */
/*=======================================================================*/

#define SD_WORD_LENGTH_5      UART_FRAME_WORD_LENGTH_5
#define SD_WORD_LENGTH_6      UART_FRAME_WORD_LENGTH_6
#define SD_WORD_LENGTH_7      UART_FRAME_WORD_LENGTH_7
#define SD_WORD_LENGTH_8      UART_FRAME_WORD_LENGTH_8

#define SD_PARITY_ODD         UART_ODD_PARITY
#define SD_PARITY_EVEN        UART_EVEN_PARITY
#define SD_PARITY_NONE        UART_PARITY_NONE

#define SD_NUM_STB_1          UART_FRAME_NUM_STB_1
#define SD_NUM_STB_1_5_2      UART_FRAME_NUM_STB_1_5_2

/*========================================================================*/
/* Driver pre-compile time settings.                                      */
/*========================================================================*/

/*
 * Configuration options
 */

/* UART0 driver enable switch */
#if !defined(AM335X_SERIAL_USE_UART0)
#define AM335X_SERIAL_USE_UART0           FALSE
#endif

/* UART0 interrupt priority level setting */
#if !defined(AM335X_SERIAL_UART0_PRIORITY)
#define AM335X_SERIAL_UART0_PRIORITY      72
#endif

/* UART0 RX/TX buffer size */
#if !defined(AM335X_SERIAL_UART0_BUFFER_SIZE)
#define AM335X_SERIAL_UART0_BUFFER_SIZE   16
#endif


/*========================================================================*/
/* Derived constants and error checks.                                    */
/*========================================================================*/

#if !AM335X_SERIAL_USE_UART0
#error "SERIAL driver activated but no USART/UART peripheral assigned";
#endif


/*========================================================================*/
/* Driver data structures and types.                                      */
/*========================================================================*/

/**
 * @brief   Generic Serial Driver configuration structure.
 * @details An instance of this structure must be passed to @p sdStart()
 *          in order to configure and start a serial driver operations.
 * @note    Implementations may extend this structure to contain more,
 *          architecture dependent, fields.
 */
typedef struct
{
  uint32_t  BaudRate;
  uint32_t  WordLength;
  uint32_t  Parity;
  uint32_t  NumStb;
} SerialConfig;

/**
 * @brief @p SerialDriver specific data.
 */
#define _serial_driver_data             \
  _base_asynchronous_channel_data       \
  /* Driver state.*/                    \
  sdstate_t                 state;      \
  /* Input queue.*/                     \
  InputQueue                iqueue;     \
  /* Output queue.*/                    \
  OutputQueue               oqueue;     \
  /* Input circular buffer.*/           \
  uint8_t                   *ib;        \
  /* Output circular buffer.*/          \
  uint8_t                   *ob;        \
  /* End of the mandatory fields.*/     \
  /* circular buffer. */                \
  uint16_t                  size;       \
  uint32_t                  base;

/*========================================================================*/
/* Driver macros.                                                         */
/*========================================================================*/

/*========================================================================*/
/* External declarations.                                                 */
/*========================================================================*/

/** @brief UART0 serial driver identifier. */
#if AM335X_SERIAL_USE_UART0 && !defined(__DOXYGEN__)
extern SerialDriver SD0;
#endif


#ifdef __cplusplus
extern "C" {
#endif
  void sd_lld_init(void);
  void sd_lld_start(SerialDriver *sdp, const SerialConfig *config);
  void sd_lld_stop(SerialDriver *sdp);
#ifdef __cplusplus
}
#endif

#endif /* HAL_USE_SERIAL */

#endif /* _SERIAL_LLD_H_ */

/** @} */
