/**************************************************************************
*  Copyright (c) 2013 by Michael Fischer. All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  09.02.2013  mifi  First Version for the BeagleBone board.
**************************************************************************/
#define __MAIN_C__

/*=======================================================================*/
/*  Include                                                              */
/*=======================================================================*/

#include <stdint.h>
#include <stdio.h>
#include <ctl_api.h>
#include <libarm.h>
#include "ch.h"
#include "hal.h"

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/


#define LED_PORT    IOPORT1

#define LED0_PAD    21
#define LED1_PAD    22
#define LED2_PAD    23
#define LED3_PAD    24

#define LED0_ON()   palSetPad(LED_PORT,LED0_PAD)
#define LED1_ON()   palSetPad(LED_PORT,LED1_PAD)
#define LED2_ON()   palSetPad(LED_PORT,LED2_PAD)
#define LED3_ON()   palSetPad(LED_PORT,LED3_PAD)

#define LED0_OFF()  palClearPad(LED_PORT,LED0_PAD)
#define LED1_OFF()  palClearPad(LED_PORT,LED1_PAD)
#define LED2_OFF()  palClearPad(LED_PORT,LED2_PAD)
#define LED3_OFF()  palClearPad(LED_PORT,LED3_PAD)

/*
 * Priority and stack size of the task.
 */
#define TASK_HP_PRIORITY   NORMALPRIO + 3
#define TASK_LP_PRIORITY   NORMALPRIO + 2

#define TASK_HP_STK_SIZE   128
#define TASK_LP_STK_SIZE   128

/*=======================================================================*/
/*  Prototypes                                                           */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

static WORKING_AREA(HPStack, TASK_HP_STK_SIZE);
static WORKING_AREA(LPStack, TASK_LP_STK_SIZE);

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/***************************************************************************/
/*  __putchar                                                              */
/*                                                                         */
/*  Support for printf to output the character over the serial line.       */
/*                                                                         */
/*  In    : ch                                                             */
/*  Out   : none                                                           */
/*  Return: ch                                                             */
/***************************************************************************/
int __putchar (int ch)
{
  chIOPut(&SD0, ch);
  
  return(ch);
} /* __putchar */

/***************************************************************************/
/*  HPTask                                                                 */
/*                                                                         */
/*  In    : task parameter                                                 */
/*  Out   : none                                                           */
/*  Return: never                                                          */
/***************************************************************************/
static msg_t HPTask (void *p)
{
  (void)p;
   
  while(1)
  {
    LED2_ON();
    chThdSleepMilliseconds(100);
    LED2_OFF();
    chThdSleepMilliseconds(100);
  }
} /* HPTask */

/***************************************************************************/
/*  LPTask                                                                 */
/*                                                                         */
/*  In    : task parameter                                                 */
/*  Out   : none                                                           */
/*  Return: never                                                          */
/***************************************************************************/
static msg_t LPTask (void *p)
{
  (void)p;
   
  while(1)
  {
    palSetPad(LED_PORT, LED1_PAD);
    chThdSleepMilliseconds(250);
    
    palClearPad(LED_PORT, LED1_PAD);
    chThdSleepMilliseconds(250);
  }
} /* LPTask */

/***************************************************************************/
/*  PrintHello                                                             */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
static void PrintHello (void)
{
  printf("\r\n");
  printf("***\r\n");
  printf("*** ChibiOS/RT\r\n");
  printf("***\r\n");
  printf("*** Kernel:       %s\r\n", CH_KERNEL_VERSION);
  printf("*** Compiled:     %s - %s\r\n", __DATE__, __TIME__);
#ifdef CH_COMPILER_NAME
  printf("*** Compiler:     %s\r\n", CH_COMPILER_NAME);
#endif
  printf("*** Architecture: %s\r\n", CH_ARCHITECTURE_NAME);
#ifdef CH_CORE_VARIANT_NAME
  printf("*** Core Variant: %s\r\n", CH_CORE_VARIANT_NAME);
#endif
#ifdef CH_PORT_INFO
  printf("*** Port Info:    %s\r\n", CH_PORT_INFO);
#endif
#ifdef PLATFORM_NAME
  printf("*** Platform:     %s\r\n", PLATFORM_NAME);
#endif
#ifdef BOARD_NAME
  printf("*** Test Board:   %s\r\n", BOARD_NAME);
#endif
  printf("***\r\n");
  printf("\r\n");
} /* PrintHello */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  main                                                                 */
/*************************************************************************/
int main (void)
{
  /*
   * System initializations.
   * - HAL initialization, this also initializes the configured device
   *   drivers and performs the board-specific initializations.
   * - Kernel initialization, the main() function becomes a thread and
   *   the RTOS is active.
   */
  halInit();
  chSysInit();
  
  /*
   * Activates the serial driver 0 using the driver default configuration.
   */
  sdStart(&SD0, NULL);
  PrintHello();
      
  /*
   * Create our threads
   */
  chThdCreateStatic(HPStack, sizeof(HPStack), TASK_HP_PRIORITY, HPTask, NULL);
  chThdCreateStatic(LPStack, sizeof(LPStack), TASK_LP_PRIORITY, LPTask, NULL);
   
  while (1)
  {
    palTogglePad(LED_PORT, LED0_PAD);
    chThdSleepMilliseconds(500);
  }

  /*
   * This return here make no sense.
   * But to prevent the compiler warning:
   * "return type of 'main' is not 'int'
   * we use an int as return :O)
   */ 
  return(0);
} /* main */

/*** EOF ***/
